@@ -11,7 +11,7 @@
 
 package MCE::Core::Manager;
 
-our $VERSION = '1.507'; $VERSION = eval $VERSION;
+our $VERSION = '1.508'; $VERSION = eval $VERSION;
 
 ## Items below are folded into MCE.
 
@@ -731,20 +731,24 @@ sub _output_loop {
    $_->($self, \$_DAU_R_SOCK) for (@{ $_plugin_loop_begin });
 
    ## Call on hash function. Exit loop when workers have completed.
-   while (1) {
-      $_func = <$_DAT_R_SOCK>;
-      next unless (defined $_func);
+   {
+      local $!;
 
-      $_DAU_R_SOCK = $_channels->[ <$_DAT_R_SOCK> ];
+      while (1) {
+         $_func = <$_DAT_R_SOCK>;
+         next unless (defined $_func);
 
-      if (exists $_core_output_function{$_func}) {
-         $_core_output_function{$_func}();
+         $_DAU_R_SOCK = $_channels->[ <$_DAT_R_SOCK> ];
+
+         if (exists $_core_output_function{$_func}) {
+            $_core_output_function{$_func}();
+         }
+         elsif (exists $_plugin_function->{$_func}) {
+            $_plugin_function->{$_func}();
+         }
+
+         last unless ($self->{_total_running});
       }
-      elsif (exists $_plugin_function->{$_func}) {
-         $_plugin_function->{$_func}();
-      }
-
-      last unless ($self->{_total_running});
    }
 
    ## Call module's loop_end routine for modules plugged into MCE.
